@@ -1,4 +1,4 @@
-/* $Id: gdict-applet.c,v 1.61 2004/07/19 02:30:42 gman Exp $ */
+/* $Id: gdict-applet.c,v 1.76 2004/11/24 19:35:58 vnoel Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -14,12 +14,8 @@
 #  include <config.h>
 #endif
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <assert.h>
-
-#include <gnome.h>
-#include <libbonoboui.h>
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
 #include <panel-applet.h>
 
 #include "gdict-app.h"
@@ -27,31 +23,9 @@
 #include "gdict-pref.h"
 #include "gdict-applet.h"
 #include "gdict-pref-dialog.h"
+#include "gdict-utils.h"
 
-
-
-#define DOCKED_APPLET_WIDTH 74
-#define FLOATING_APPLET_WIDTH 14
-#define SHORT_APPLET_HEIGHT 22
-#define TALL_APPLET_HEIGHT 44
-
-/* This is a hack around the fact that gtk+ doesn't
- * propogate button presses on button2/3.
- */
-static gboolean 
-button_press_hack (GtkWidget *w, GdkEventButton *event, gpointer data)
-{
-    GtkWidget *applet = GTK_WIDGET (data);
-    
-    if (event->button == 3 || event->button == 2) {
-	gtk_propagate_event (applet, (GdkEvent *) event);
-
-	return TRUE;
-    }
-    
-    return FALSE;
-    
-}
+static const gchar *tooltip_str = N_("Lookup words in an online dictionary");
 
 static void
 entry_activate_cb (GtkEntry *entry, gpointer data)
@@ -72,22 +46,24 @@ entry_activate_cb (GtkEntry *entry, gpointer data)
     
     	if (!text || *text == 0)
         	return;
-        
-    	g_utf8_strdown (text, strlen (text));
+
 	gtk_editable_delete_text (GTK_EDITABLE (entry), 0, -1);
     	gdict_app_do_lookup (text);
     	g_free (text);
- 
-
 }
 
-static void
-button_press_cb (GtkButton *button, gpointer data)
+static gboolean
+image_clicked (GtkWidget *w, GdkEventButton *event, gpointer data)
 {
 	GDictApplet *applet = data;
-	
-	entry_activate_cb (GTK_ENTRY (applet->entry_widget), applet);
-	
+
+	if (event->button == 3 || event->button ==2) {
+		gtk_propagate_event (GTK_WIDGET(applet->applet_widget), (GdkEvent *) event);
+		return TRUE;
+	}
+
+	entry_activate_cb (GTK_ENTRY(applet->entry_widget), applet);
+	return FALSE;
 }
 
 static void
@@ -117,65 +93,103 @@ setup_applet_widgets (GDictApplet *applet)
   		{ "text/plain", 0, 0 },
   		{ "STRING",     0, 0 }
     	};
-	
+	GtkWidget *eventbox;
+
 	applet->about_dialog = NULL;
 
-	if (applet->box) {
+	if (applet->box)
 		gtk_widget_destroy (applet->box);
-	}
 
-	if ((applet->orient == PANEL_APPLET_ORIENT_LEFT || applet->orient == PANEL_APPLET_ORIENT_RIGHT) || 
-	    ((applet->orient == PANEL_APPLET_ORIENT_UP || applet->orient == PANEL_APPLET_ORIENT_DOWN) && applet->panel_size > 48)) {
+	if (applet->orient == PANEL_APPLET_ORIENT_LEFT || applet->orient == PANEL_APPLET_ORIENT_RIGHT)
 		applet->box = gtk_vbox_new (FALSE, 2);
-	}
-	else {
+	else
 		applet->box = gtk_hbox_new (FALSE, 2);
-	} 
 	
 	gtk_container_set_border_width (GTK_CONTAINER (applet->box), 0);
 
-	applet->button_widget = gtk_button_new_with_mnemonic (_("_Lookup"));
-
-	gtk_drag_dest_set (applet->button_widget, GTK_DEST_DEFAULT_ALL, drop_targets, 
+	eventbox = gtk_event_box_new ();
+	gtk_event_box_set_visible_window (GTK_EVENT_BOX(eventbox), FALSE);
+	applet->image_widget = gtk_image_new_from_icon_name ("gdict", applet->panel_size);
+	gtk_container_add (GTK_CONTAINER (eventbox), applet->image_widget);
+	gtk_tooltips_set_tip (applet->tooltips, eventbox, _(tooltip_str), NULL);
+	gtk_drag_dest_set (eventbox, GTK_DEST_DEFAULT_ALL, drop_targets, 
     		           G_N_ELEMENTS (drop_targets), GDK_ACTION_COPY | GDK_ACTION_MOVE);
-	gtk_box_pack_end (GTK_BOX (applet->box), applet->button_widget, TRUE, TRUE, 0);
+
+	gtk_box_pack_start (GTK_BOX (applet->box), eventbox, TRUE, TRUE, 0);
 	
-        if (GTK_IS_ACCESSIBLE (gtk_widget_get_accessible (applet->button_widget)))          
-        add_atk_namedesc (GTK_WIDGET (applet->button_widget), _("Dictionary Lookup"), NULL);
+        if (GTK_IS_ACCESSIBLE (gtk_widget_get_accessible (applet->image_widget)))          
+        gdict_add_atk_namedesc (GTK_WIDGET (applet->image_widget),
+				_("Dictionary Lookup"), NULL);
 
 	applet->entry_widget = gtk_entry_new ();
 	if (GTK_IS_ACCESSIBLE (gtk_widget_get_accessible (applet->entry_widget)))
-		add_atk_namedesc (GTK_WIDGET (applet->entry_widget), _("Dictionary word entry"), NULL);
+		gdict_add_atk_namedesc (GTK_WIDGET (applet->entry_widget),
+					_("Dictionary word entry"), NULL);
 
 	gtk_entry_set_width_chars (GTK_ENTRY (applet->entry_widget), 12);
-	gtk_box_pack_start (GTK_BOX (applet->box), applet->entry_widget, FALSE, FALSE, 0);
+	gtk_tooltips_set_tip (applet->tooltips, applet->entry_widget, _(tooltip_str), NULL);
+	gtk_box_pack_end (GTK_BOX (applet->box), applet->entry_widget, TRUE, TRUE, 0);
 
  	gtk_widget_show_all (applet->box);
-	  	
- 	gtk_container_add(GTK_CONTAINER(applet->applet_widget), applet->box);                              
+
+ 	gtk_container_add (GTK_CONTAINER(applet->applet_widget), applet->box);                              
 
  	gtk_widget_show (applet->applet_widget);
 
-	gtk_tooltips_set_tip (GTK_TOOLTIPS(applet->tooltip_widget), applet->entry_widget,
-			      _("Lookup words in an online dictionary"), NULL);
-
  	g_signal_connect (G_OBJECT (applet->entry_widget), "activate",
  			  G_CALLBACK (entry_activate_cb), applet);
-			  
-	g_signal_connect (G_OBJECT (applet->button_widget), "button_press_event",
-			  G_CALLBACK (button_press_hack), applet->applet_widget);
-	g_signal_connect (G_OBJECT (applet->button_widget), "clicked",
- 			  G_CALLBACK (button_press_cb), applet);
- 	g_signal_connect (G_OBJECT (applet->button_widget), "drag_data_received",
+	g_signal_connect (G_OBJECT (eventbox), "button_release_event",
+ 			  G_CALLBACK (image_clicked), applet);
+ 	g_signal_connect (G_OBJECT (eventbox), "drag_data_received",
     		          G_CALLBACK (lookup_button_drag_cb), applet);
 }
 
-static void
-change_size_cb (PanelApplet *widget, gint size, GDictApplet *applet)
-{
-	applet->panel_size = size;
-	setup_applet_widgets (applet);
 
+static void
+change_background (PanelApplet *applet,
+		   PanelAppletBackgroundType type,
+		   GdkColor *colour,
+		   GdkPixmap *pixmap)
+{
+	GtkRcStyle *rc_style;
+	GtkStyle *style;
+
+	/* reset style */
+	gtk_widget_set_style (GTK_WIDGET (applet), NULL);
+	rc_style = gtk_rc_style_new ();
+	gtk_widget_modify_style (GTK_WIDGET (applet), rc_style);
+	g_object_unref (rc_style);
+
+	switch (type) {
+	case PANEL_NO_BACKGROUND:
+		break;
+	case PANEL_COLOR_BACKGROUND:
+		gtk_widget_modify_bg (GTK_WIDGET (applet),
+				      GTK_STATE_NORMAL, colour);
+		break;
+	case PANEL_PIXMAP_BACKGROUND:
+		style = gtk_style_copy (GTK_WIDGET (applet)->style);
+		if (style->bg_pixmap[GTK_STATE_NORMAL])
+			g_object_unref (style->bg_pixmap[GTK_STATE_NORMAL]);
+		style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref (pixmap);
+		gtk_widget_set_style (GTK_WIDGET (applet), style);
+		break;
+	}
+}
+
+static void
+size_allocate_cb (PanelApplet *widget, GtkAllocation *allocation, GDictApplet *applet)
+{
+	if (applet->orient == PANEL_APPLET_ORIENT_UP || applet->orient == PANEL_APPLET_ORIENT_DOWN) {
+		if (applet->panel_size == allocation->height)
+			return;
+		applet->panel_size = allocation->height;
+	} else {
+		if (applet->panel_size == allocation->width)
+			return;
+		applet->panel_size = allocation->width;
+	}
+	setup_applet_widgets (applet);
 }
 
 static void
@@ -183,7 +197,6 @@ change_orient_cb (PanelApplet *widget, PanelAppletOrient orient, GDictApplet *ap
 {
 	applet->orient = orient;
 	setup_applet_widgets (applet);
-	
 }
 
 static gboolean
@@ -202,10 +215,10 @@ set_show_dictionary_menu_state (GtkWidget *widget, GdkEventButton *event, gpoint
 					      "/commands/def win",
 					      "state",
 					      GTK_WIDGET_VISIBLE (gdict_app) ? "1" : "0",
-				 	      NULL); 
-				      
-    }
-    return FALSE;
+				 	      NULL);
+	}
+
+	return FALSE;
 }
                                                                                 
 static gboolean
@@ -215,6 +228,7 @@ send_button_to_entry_event (GtkWidget *widget, GdkEventButton *event, GDictApple
 		gtk_widget_grab_focus (applet->entry_widget);
 		return TRUE;
 	}
+
 	return FALSE;
 }
 
@@ -226,9 +240,9 @@ destroy_applet (GtkWidget *widget, gpointer data)
 	if (applet->about_dialog)
 		gtk_widget_destroy (applet->about_dialog);
 
-	gtk_widget_destroy (applet->button_widget);
-	gtk_widget_destroy (applet->box);
+	gtk_widget_destroy (applet->image_widget);
 	gtk_widget_destroy (applet->entry_widget);
+	gtk_widget_destroy (applet->box);
 	gtk_widget_destroy (applet->applet_widget);
 }
 
@@ -270,7 +284,7 @@ text_received (GtkClipboard *clipboard, const gchar *text, gpointer data)
 	
 	if (*lookup_text == 0)
         	return;
-    	g_utf8_strdown (lookup_text, strlen(lookup_text));
+
     	gtk_entry_set_text (GTK_ENTRY (applet->entry_widget), text);
     	entry_activate_cb (GTK_ENTRY (applet->entry_widget), applet);
 #if 0    	
@@ -278,8 +292,6 @@ text_received (GtkClipboard *clipboard, const gchar *text, gpointer data)
     	gtk_entry_set_text (GTK_ENTRY (word_entry), lookup_text);
 #endif
     	g_free (lookup_text);
- 
-	
 }
 
 static void
@@ -292,6 +304,7 @@ lookup_selected_text (BonoboUIComponent *uic, gpointer data, const gchar *verbna
  	gtk_clipboard_request_text (clipboard, text_received, applet);
  	
 }
+
 static void
 about_cb (BonoboUIComponent *uic,
           gpointer           user_data,
@@ -312,6 +325,8 @@ about_cb (BonoboUIComponent *uic,
 	gtk_window_set_screen (GTK_WINDOW (applet->about_dialog),
 			       gtk_widget_get_screen (applet->applet_widget));
 
+	g_signal_connect (G_OBJECT (applet->about_dialog), "response",
+			  G_CALLBACK (gtk_widget_destroy), NULL);
 	g_signal_connect (G_OBJECT (applet->about_dialog), "destroy",
 			  G_CALLBACK (gtk_widget_destroyed), &applet->about_dialog);
 
@@ -326,16 +341,14 @@ help_cb (BonoboUIComponent *uic,
 	GError *error = NULL;
 	gnome_help_display("gnome-dictionary",NULL,&error);
 
-	if (error)
-	{
+	if (error) {
 		GtkWidget *dialog;
 
-		dialog = gtk_message_dialog_new (NULL,
-						 GTK_DIALOG_MODAL,
-						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_OK,
-						 ("There was an error displaying help: \n%s"),
-						 error->message);
+		dialog = gtk_message_dialog_new_with_markup (NULL, GTK_DIALOG_MODAL, 
+							     GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+							     "<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
+							     _("Could not display help"),
+							     error->message);
 
 		g_signal_connect (G_OBJECT (dialog), "response",
 				  G_CALLBACK (gtk_widget_destroy),
@@ -373,7 +386,6 @@ gdict_applet_new (PanelApplet *parent_applet)
 	GDictApplet *applet;
 	BonoboUIComponent *component;
 	GtkIconInfo *icon_info;
-	GtkTooltips *tooltips;
 
  	applet = g_new0 (GDictApplet, 1);
  	
@@ -381,19 +393,13 @@ gdict_applet_new (PanelApplet *parent_applet)
 	applet->panel_size = panel_applet_get_size (PANEL_APPLET (applet->applet_widget));
  	applet->orient = panel_applet_get_orient (PANEL_APPLET (applet->applet_widget));
 	
-	icon_info = gtk_icon_theme_lookup_icon (gtk_icon_theme_get_default (), "gdict", 48, 0);
-	if (icon_info) {
-		gnome_window_icon_set_default_from_file (gtk_icon_info_get_filename (icon_info));
-		gtk_icon_info_free (icon_info);
-	}
+	gtk_window_set_default_icon_name ("gdict");
  	
 	panel_applet_set_flags (parent_applet, PANEL_APPLET_EXPAND_MINOR);
 
-	applet->tooltip_widget = GTK_WIDGET(gtk_tooltips_new ());
-	g_object_ref (applet->tooltip_widget);
-	gtk_object_sink (GTK_OBJECT (applet->tooltip_widget));
-	g_object_set_data_full (G_OBJECT (applet->applet_widget), "tooltips",
-				applet->tooltip_widget, (GDestroyNotify) g_object_unref);
+	applet->tooltips = gtk_tooltips_new ();
+	g_object_ref (applet->tooltips);
+	gtk_object_sink (GTK_OBJECT (applet->tooltips));
 
 	setup_applet_widgets (applet);
 
@@ -425,8 +431,8 @@ gdict_applet_new (PanelApplet *parent_applet)
 	component = panel_applet_get_popup_component ((PanelApplet *)applet->applet_widget);
 	g_signal_connect (component, "ui-event", (GCallback) menu_event_cb, NULL);
 	
- 	g_signal_connect (G_OBJECT (applet->applet_widget), "change_size",
- 			  G_CALLBACK (change_size_cb), applet);
+ 	g_signal_connect (G_OBJECT (applet->applet_widget), "size_allocate",
+ 			  G_CALLBACK (size_allocate_cb), applet);
  	g_signal_connect (G_OBJECT (applet->applet_widget), "change_orient",
  			  G_CALLBACK (change_orient_cb), applet);
  	g_signal_connect (G_OBJECT (applet->applet_widget), "button_press_event",                         
@@ -435,6 +441,8 @@ gdict_applet_new (PanelApplet *parent_applet)
 			  G_CALLBACK (set_show_dictionary_menu_state), NULL);
 	g_signal_connect (G_OBJECT (applet->applet_widget), "destroy",
 			  G_CALLBACK (destroy_applet), applet);
+	g_signal_connect (G_OBJECT (applet->applet_widget), "change_background",
+			  G_CALLBACK (change_background), applet);
 
 	return TRUE;
 }
