@@ -1,4 +1,4 @@
-/* $Id: gdict-pref-dialog.c,v 1.41 2004/05/08 01:57:09 gman Exp $ */
+/* $Id: gdict-pref-dialog.c,v 1.45 2004/11/24 22:15:27 vnoel Exp $ */
 
 /*
  *  Mike Hughes <mfh@psilord.com>
@@ -15,17 +15,13 @@
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <assert.h>
-#include <string.h>
-#include <ctype.h>
-
-#include <gnome.h>
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
+
 #include "gdict-pref-dialog.h"
 #include "gdict-app.h"
+#include "gdict-utils.h"
 
 static void gdict_pref_dialog_class_init (GDictPrefDialogClass *class);
 static void gdict_pref_dialog_init (GDictPrefDialog *pref_dialog);
@@ -73,12 +69,6 @@ GtkWidget *alignment1 = NULL;
 GtkWidget *alignment2 = NULL;
 
 
-/* gdict_pref_dialog_get_type
- *
- * Register the GDictPrefDialog type with Gtk's type system if necessary and
- * return the type identifier code
- */
-
 GType
 gdict_pref_dialog_get_type (void)
 {
@@ -108,13 +98,6 @@ gdict_pref_dialog_get_type (void)
     return gdict_pref_dialog_type;
 }
 
-/* gdict_pref_dialog_class_init
- *
- * Initialises a structure describing the GDictPrefDialog class; sets
- * up signals for pref_dialog events in the Gtk signal management
- * system
- */
-
 static void 
 gdict_pref_dialog_class_init (GDictPrefDialogClass *class) 
 {
@@ -137,16 +120,11 @@ gdict_pref_dialog_class_init (GDictPrefDialogClass *class)
     object_class->finalize = gdict_pref_dialog_finalize;
 }
 
-/* gdict_pref_dialog_init
- *
- * Initialises an instance of a GDictPrefDialog object
- */
-
 static void 
 gdict_pref_dialog_init (GDictPrefDialog *pref_dialog)
 {
     GtkWidget *server_label, *port_label;
-    GtkWidget *vbox;
+    GtkWidget *vbox, *hbox;
     GtkWidget *button;    
     gchar *port_str;
     
@@ -156,12 +134,12 @@ gdict_pref_dialog_init (GDictPrefDialog *pref_dialog)
     pref_dialog->dfl_strat = gdict_pref.dfl_strat;
     
     gtk_window_set_title (GTK_WINDOW (pref_dialog), _("Dictionary Preferences"));
-    gtk_window_set_policy (GTK_WINDOW (pref_dialog), FALSE, FALSE, FALSE);    
+    gtk_window_set_resizable (GTK_WINDOW (pref_dialog), FALSE);
     gtk_container_set_border_width (GTK_CONTAINER (pref_dialog), 5);
     vbox = GTK_DIALOG (pref_dialog)->vbox;
     gtk_box_set_spacing (GTK_BOX (vbox), 2);
     
-    pref_dialog->table = GTK_TABLE (gtk_table_new (5, 3, FALSE));
+    pref_dialog->table = GTK_TABLE (gtk_table_new (5, 2, FALSE));
     gtk_box_pack_start (GTK_BOX (GTK_DIALOG (pref_dialog)->vbox), GTK_WIDGET (pref_dialog->table), TRUE, TRUE, 0);
     gtk_container_set_border_width (GTK_CONTAINER (pref_dialog->table), 5);
     gtk_table_set_row_spacings (pref_dialog->table, 6);
@@ -172,20 +150,27 @@ gdict_pref_dialog_init (GDictPrefDialog *pref_dialog)
     g_signal_connect (G_OBJECT (pref_dialog->smart_lookup_btn), "toggled",
     		      G_CALLBACK (smart_lookup_cb), pref_dialog);
     gtk_table_attach_defaults (pref_dialog->table, GTK_WIDGET (pref_dialog->smart_lookup_btn),
-    			       1, 3, 2, 3);
+    			       1, 2, 2, 3);
     if ( ! gconf_client_key_is_writable (gdict_get_gconf_client (), "/apps/gnome-dictionary/smart", NULL))
 	    gtk_widget_set_sensitive (GTK_WIDGET (pref_dialog->smart_lookup_btn), FALSE);
     
-    pref_dialog->server_entry = GTK_ENTRY (gtk_entry_new ());			       
+    hbox = gtk_hbox_new (FALSE, 6);
+    pref_dialog->server_entry = GTK_ENTRY (gtk_entry_new ());	
+    gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET(pref_dialog->server_entry), TRUE, TRUE, 0);
+    g_signal_connect (G_OBJECT (pref_dialog->server_entry), "focus_out_event",
+    		      G_CALLBACK (set_server_cb), pref_dialog);
+
+    button = gtk_button_new_with_mnemonic (_("De_fault Server"));
+    gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET(button), FALSE, FALSE, 0);
+    g_signal_connect (G_OBJECT (button), "clicked",
+    		      G_CALLBACK (set_default_server), pref_dialog);
+
     gtk_table_attach (pref_dialog->table, 
-                      GTK_WIDGET (pref_dialog->server_entry),
-                      1, 2, 0, 1,
+		      hbox,
+		      1, 2, 0, 1,
                       (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                       (GtkAttachOptions) (0), 0, 0);
 		    
-    g_signal_connect (G_OBJECT (pref_dialog->server_entry), "focus_out_event",
-    		      G_CALLBACK (set_server_cb), pref_dialog);
-    
     server_label = gtk_label_new_with_mnemonic (_("_Server:"));
     gtk_label_set_mnemonic_widget (GTK_LABEL (server_label), 
     				   GTK_WIDGET (pref_dialog->server_entry));
@@ -193,16 +178,9 @@ gdict_pref_dialog_init (GDictPrefDialog *pref_dialog)
     gtk_misc_set_alignment (GTK_MISC (server_label), 0, 0.5);
     gtk_table_attach_defaults (pref_dialog->table, server_label, 0, 1, 0, 1);
     
-    button = gtk_button_new_with_mnemonic (_("De_fault Server"));
-    gtk_table_attach (pref_dialog->table, 
-                      button, 2, 3, 0, 1,
-                      (GtkAttachOptions) (GTK_FILL),
-                      (GtkAttachOptions) (0), 0, 0);
-    g_signal_connect (G_OBJECT (button), "clicked",
-    		      G_CALLBACK (set_default_server), pref_dialog);
 
     if (gail_loaded)
-        add_atk_namedesc(GTK_WIDGET(button), NULL, _("Reset server to default"));
+        gdict_add_atk_namedesc (GTK_WIDGET (button), NULL, _("Reset server to default"));
 
     if ( ! gconf_client_key_is_writable (gdict_get_gconf_client (), "/apps/gnome-dictionary/server", NULL)) {
 	    gtk_widget_set_sensitive (GTK_WIDGET (pref_dialog->server_entry), FALSE);
@@ -211,36 +189,37 @@ gdict_pref_dialog_init (GDictPrefDialog *pref_dialog)
     }
 
     port_label = gtk_label_new_with_mnemonic (_("_Port:"));
-    gtk_table_attach_defaults (pref_dialog->table, port_label, 0, 1, 1, 2);
     gtk_label_set_justify (GTK_LABEL (port_label), GTK_JUSTIFY_LEFT);
     gtk_misc_set_alignment (GTK_MISC (port_label), 0, 0.5);
+    gtk_table_attach_defaults (pref_dialog->table, port_label, 0, 1, 1, 2);
 
     pref_dialog->port_entry = GTK_ENTRY (gtk_entry_new ());
     gtk_label_set_mnemonic_widget (GTK_LABEL (port_label), 
     				   GTK_WIDGET (pref_dialog->port_entry));
-    gtk_table_attach (pref_dialog->table, 
-                      GTK_WIDGET (pref_dialog->port_entry),
-                      1, 2, 1, 2,
-                      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
-                      (GtkAttachOptions) (0), 0, 0);
     g_signal_connect (G_OBJECT (pref_dialog->port_entry), "focus_out_event",
     		      G_CALLBACK (set_port_cb), pref_dialog);
-
     button = gtk_button_new_with_mnemonic (_("Def_ault Port"));
-    gtk_table_attach (pref_dialog->table, 
-                      button, 2, 3, 1, 2,
-                      (GtkAttachOptions) (GTK_FILL),
-                      (GtkAttachOptions) (0), 0, 0);
     g_signal_connect (G_OBJECT (button), "clicked",
     		      G_CALLBACK (set_default_port), pref_dialog);
 
-    if (gail_loaded) {
-        add_atk_namedesc(GTK_WIDGET(button), NULL, _("Reset port to default"));
-        add_atk_namedesc(server_label, _("Server"), _("Server"));
-        add_atk_namedesc( GTK_WIDGET (pref_dialog->server_entry), _("Server Entry"), _("Enter the Server Name"));
+    hbox = gtk_hbox_new (FALSE, 6);
+    gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET(pref_dialog->port_entry), TRUE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+    gtk_table_attach (pref_dialog->table, 
+                      hbox,
+                      1, 2, 1, 2,
+                      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+                      (GtkAttachOptions) (0), 0, 0);
 
-        add_atk_namedesc(port_label, _("Port"), _("Port"));
-        add_atk_namedesc( GTK_WIDGET(pref_dialog->port_entry), _("Port Entry"), _("Enter the Port Number"));
+
+    if (gail_loaded) {
+        gdict_add_atk_namedesc (GTK_WIDGET(button), NULL, _("Reset port to default"));
+        gdict_add_atk_namedesc (server_label, _("Server"), _("Server"));
+        gdict_add_atk_namedesc (GTK_WIDGET (pref_dialog->server_entry),
+				_("Server Entry"), _("Enter the Server Name"));
+        gdict_add_atk_namedesc (port_label, _("Port"), _("Port"));
+        gdict_add_atk_namedesc (GTK_WIDGET (pref_dialog->port_entry),
+				_("Port Entry"), _("Enter the Port Number"));
     }
 
     if ( ! gconf_client_key_is_writable (gdict_get_gconf_client (), "/apps/gnome-dictionary/port", NULL)) {
@@ -290,11 +269,6 @@ gdict_pref_dialog_init (GDictPrefDialog *pref_dialog)
     gtk_widget_show_all (GTK_WIDGET (pref_dialog));
 }
 
-/* gdict_pref_dialog_finalize
- *
- * Finalize a GDictPrefDialog object
- */
-
 static void
 gdict_pref_dialog_finalize (GObject *object)
 {
@@ -309,11 +283,6 @@ gdict_pref_dialog_finalize (GObject *object)
     G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-/* gdict_pref_dialog_new
- *
- * Creates a new GDictPrefDialog object
- */
-
 GtkWidget *
 gdict_pref_dialog_new (dict_context_t *context) 
 {
@@ -601,7 +570,7 @@ pref_status_cb (dict_command_t *command, DictStatusCode code,
         gtk_option_menu_set_menu (option_menu, GTK_WIDGET (use_menu));
 
         gtk_table_attach_defaults (pref_dialog->table, GTK_WIDGET (option_menu), 
-                                   1, 3, row, row + 1);
+                                   1, 2, row, row + 1);
         gtk_widget_show_all (GTK_WIDGET (pref_dialog->table));
         
         if (command->cmd == C_SHOW_DB) {
@@ -610,12 +579,13 @@ pref_status_cb (dict_command_t *command, DictStatusCode code,
                                            GTK_WIDGET (pref_dialog->db_sel));
 
             if (gail_loaded) {
-                add_atk_namedesc(pref_dialog->db_label, _("Database"), _("Database Name"));
-                add_atk_relation( GTK_WIDGET (option_menu), pref_dialog->db_label, ATK_RELATION_LABELLED_BY);
-
+		gdict_add_atk_namedesc (pref_dialog->db_label, _("Database"), _("Database Name"));
+		gdict_add_atk_relation (GTK_WIDGET (option_menu),
+					pref_dialog->db_label, ATK_RELATION_LABELLED_BY);
             }
 
-            writable = gconf_client_key_is_writable (gdict_get_gconf_client (), "/apps/gnome-dictionary/database", NULL);
+            writable = gconf_client_key_is_writable (gdict_get_gconf_client (),
+						     "/apps/gnome-dictionary/database", NULL);
 
 	    gtk_widget_set_sensitive (pref_dialog->db_label, writable);
 	    gtk_widget_set_sensitive (GTK_WIDGET (option_menu), writable);
@@ -626,9 +596,10 @@ pref_status_cb (dict_command_t *command, DictStatusCode code,
                                            GTK_WIDGET (pref_dialog->strat_sel));
 
             if (gail_loaded) {
-                add_atk_namedesc(pref_dialog->strat_label, _("Default Strategy"), _("Default Strategy"));
-                add_atk_relation( GTK_WIDGET (option_menu), pref_dialog->strat_label, ATK_RELATION_LABELLED_BY);
-
+                gdict_add_atk_namedesc (pref_dialog->strat_label,
+					_("Default Strategy"), _("Default Strategy"));
+                gdict_add_atk_relation (GTK_WIDGET (option_menu),
+					pref_dialog->strat_label, ATK_RELATION_LABELLED_BY);
             }
 
             writable = gconf_client_key_is_writable (gdict_get_gconf_client (), "/apps/gnome-dictionary/strategy", NULL);
@@ -645,16 +616,18 @@ response_cb (GtkDialog *dialog, gint response)
     if (response == GTK_RESPONSE_HELP) {
         GError *error = NULL;
 
-        gnome_help_display ("gnome-dictionary", "gdict-settings", &error);
+        gnome_help_display_on_screen ("gnome-dictionary", "gdict-settings",
+				      gtk_widget_get_screen (GTK_WIDGET (dialog)), &error);
         if (error) {
             GtkWidget *msg_dialog;
 
-            msg_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog),
-                                                 GTK_DIALOG_DESTROY_WITH_PARENT,
-                                                 GTK_MESSAGE_ERROR,
-                                                 GTK_BUTTONS_OK,
-                                                 _("There was an error displaying help: \n%s"),
-                                                 error->message);
+            msg_dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW (dialog),
+							     GTK_DIALOG_DESTROY_WITH_PARENT,
+							     GTK_MESSAGE_ERROR,
+							     GTK_BUTTONS_OK,
+							      "<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
+							     _("Could not display help"),
+							     error->message);
 
             g_signal_connect (G_OBJECT (msg_dialog), "response",
                               G_CALLBACK (gtk_widget_destroy), NULL);
