@@ -405,9 +405,8 @@ open_file_cb (GtkWidget 	*widget,
 				
 				if (launch_file (locale_file) == FALSE) {
 					
-					if ((is_nautilus_running ()) &&
-					    ((g_file_test (locale_file, G_FILE_TEST_IS_DIR)) ||
-					     (is_component_action_type (locale_file)))) {
+					if (is_nautilus_running () &&
+					    g_file_test (locale_file, G_FILE_TEST_IS_DIR)) {
 						open_file_with_nautilus (locale_file);
 					}
 					else {
@@ -988,9 +987,8 @@ file_event_after_cb  (GtkWidget 	*widget,
 				
 				if (launch_file (locale_file) == FALSE) {
 				
-					if ((is_nautilus_running ()) &&
-					    ((g_file_test (locale_file, G_FILE_TEST_IS_DIR)) ||
-					     (is_component_action_type (locale_file)))) {
+					if (is_nautilus_running () &&
+					    g_file_test (locale_file, G_FILE_TEST_IS_DIR)) {
 						open_file_with_nautilus (locale_file);
 					}
 					else {
