@@ -126,10 +126,6 @@ gboolean
 key_press_cb 			(GtkWidget 	*widget, 
 				 GdkEventKey 	*event,
 				 gpointer 	data);	
-gboolean    
-look_in_folder_key_press_cb 	(GtkWidget 	*widget, 
-				 GdkEventKey    *event,
-				 gpointer 	data);
 				 
 gboolean
 file_button_release_event_cb	(GtkWidget 	*widget,
@@ -151,6 +147,11 @@ file_key_press_event_cb		(GtkWidget 	*widget,
 gboolean
 not_running_timeout_cb 		(gpointer data);
 
+void
+disable_quick_search_cb         (GtkWidget 	*dialog, 
+				 gint      	response, 
+				 gpointer  	data);
+
 #ifdef __cplusplus
 }
 #endif
