@@ -265,8 +265,37 @@ setup_find_name_options (gchar *filename)
 	return g_string_free (command, FALSE);
 }
 
+gboolean
+has_additional_constraints (void)
+{
+	GList *list;
+	
+	if (interface.selected_constraints != NULL) {
+	
+		for (list = interface.selected_constraints; list != NULL; list = g_list_next (list)) {
+			
+			SearchConstraint *constraint = list->data;
+							
+			switch (templates[constraint->constraint_id].type) {
+			case SEARCH_CONSTRAINT_BOOL:
+			case SEARCH_CONSTRAINT_NUMBER:
+			case SEARCH_CONSTRAINT_TIME_LESS:	
+			case SEARCH_CONSTRAINT_TIME_MORE:
+				return TRUE;
+			case SEARCH_CONSTRAINT_TEXT:
+				if (strlen (constraint->data.text) > 0) {
+					return TRUE;
+				} 
+			default:
+				break;
+			}
+		}
+	}
+	return FALSE;
+}
+
 gchar *
-build_search_command (void)
+build_search_command (gboolean first_pass)
 {
 	GString *command;
 	gchar *file_is_named_utf8;
@@ -302,10 +331,8 @@ build_search_command (void)
 	}
 	file_is_named_locale = g_locale_from_utf8 (file_is_named_utf8, -1, NULL, NULL, NULL);
 	
-	look_in_folder_utf8 = gnome_file_entry_get_full_path (GNOME_FILE_ENTRY(interface.look_in_folder_entry), FALSE);
+	look_in_folder_utf8 = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (interface.look_in_folder_entry));
 	look_in_folder_locale = g_locale_from_utf8 (look_in_folder_utf8, -1, NULL, NULL, NULL);
-	gnome_entry_prepend_history (GNOME_ENTRY(gnome_file_entry_gnome_entry (GNOME_FILE_ENTRY(interface.look_in_folder_entry))), 
-				     TRUE, look_in_folder_utf8);
 	g_free (look_in_folder_utf8);
 	
 	if (!file_extension_is (look_in_folder_locale, G_DIR_SEPARATOR_S)) {
@@ -319,56 +346,57 @@ build_search_command (void)
 	
 	command = g_string_new ("");
 	search_command.show_hidden_files = FALSE;
-	search_command.quick_mode = FALSE;
 	search_command.regex_string = NULL;
 	search_command.date_format_pref = NULL;
 	search_command.file_is_named_pattern = NULL;
 	
+	search_command.first_pass = first_pass;
+	if (search_command.first_pass == TRUE) {
+		search_command.quick_mode = FALSE;
+	}
+	
 	if ((GTK_WIDGET_VISIBLE(interface.additional_constraints) == FALSE) ||
-	    (interface.selected_constraints == NULL)) {
+	    (has_additional_constraints() == FALSE)) {
 	
-		gchar *locate;
-		gboolean disable_quick_search;
-		gboolean force_quick_search;
-		
-		locate = g_find_program_in_path ("locate");
 		file_is_named_backslashed = backslash_special_characters (file_is_named_locale);
 		file_is_named_escaped = escape_single_quotes (file_is_named_backslashed);
-
 		search_command.file_is_named_pattern = g_strdup(file_is_named_utf8);
-		disable_quick_search = gsearchtool_gconf_get_boolean ("/apps/gnome-search-tool/disable_quick_search");
-		force_quick_search = gsearchtool_gconf_get_boolean ("/apps/gnome-search-tool/force_quick_search");
 		
-		if ((disable_quick_search != TRUE) && (locate != NULL)) {
-		    	if ((force_quick_search == TRUE) || 
-			    ((is_path_in_home_folder (look_in_folder_locale) != TRUE) 
-			     && (is_path_in_mount_folder (look_in_folder_locale) != TRUE)
-			     && (is_path_in_proc_folder (look_in_folder_locale) != TRUE) 
-			     && (is_path_in_dev_folder (look_in_folder_locale) != TRUE)
-			     && (is_path_in_var_folder (look_in_folder_locale) != TRUE)
-			     && (is_path_in_tmp_folder (look_in_folder_locale) != TRUE))) {	
-				g_string_append_printf (command, "%s %s '%s*%s'", 
+		if (search_command.first_pass == TRUE) {
+		
+			gchar *locate;
+			gboolean disable_quick_search;
+		
+			locate = g_find_program_in_path ("locate");
+			disable_quick_search = gsearchtool_gconf_get_boolean ("/apps/gnome-search-tool/disable_quick_search");		
+			search_command.disable_second_pass = gsearchtool_gconf_get_boolean ("/apps/gnome-search-tool/disable_quick_search_second_pass");
+		
+			if ((disable_quick_search == FALSE) 
+			    && (locate != NULL) 
+			    && (is_quick_search_excluded_path (look_in_folder_locale) == FALSE)) {
+			    
+					g_string_append_printf (command, "%s %s '%s*%s'", 
 							locate,
 							locate_command_default_options,
 							look_in_folder_locale,
 							file_is_named_escaped);
-				search_command.quick_mode = TRUE;
-		 	}
-			else {
+					search_command.quick_mode = TRUE;
+			}
+			else {		
 				g_string_append_printf (command, "find \"%s\" %s '%s' -xdev -print", 
 							look_in_folder_locale, 
 							find_command_default_name_option, 
 							file_is_named_escaped);
 			}
-		} 
+			g_free (locate);
+		}
 		else {
 			g_string_append_printf (command, "find \"%s\" %s '%s' -xdev -print", 
 						look_in_folder_locale, 
 						find_command_default_name_option, 
 						file_is_named_escaped);
 		}
-		g_free (locate);
-	} 
+	}
 	else {
 		GList *list;
 		gboolean disable_mount_argument = FALSE;
@@ -492,6 +520,12 @@ add_file_to_search_results (const gchar 	*file,
 	gchar *utf8_base_name, *utf8_dir_name;
 	gchar *base_name, *dir_name;
 	
+	if (g_hash_table_lookup_extended (search_command.file_hash, file, NULL, NULL) == TRUE) {
+		return;
+	}
+	
+	g_hash_table_insert (search_command.file_hash, g_strdup(file), NULL);
+	
 	mime_type = gnome_vfs_get_file_mime_type (file, NULL, FALSE);
 	 
 	description = get_file_type_for_mime_type (file, mime_type);
@@ -608,6 +642,26 @@ update_search_counts (void)
 	g_free (stopped_string);
 }
 
+static void
+intermediate_file_count_update (void)
+{
+	gchar *string;
+	gint  count;
+	
+	count = gtk_tree_model_iter_n_children (GTK_TREE_MODEL(interface.model), NULL);
+	
+	if (count > 0) {
+		
+		string = g_strdup_printf (ngettext ("%d file found",
+		                                    "%d files found",
+		                                    count),
+		                          count);
+					  
+		gtk_label_set_text (GTK_LABEL (interface.results_label), string);
+		g_free (string);
+	}
+}
+
 static GtkWidget *
 make_list_of_templates (void)
 {
@@ -864,7 +918,7 @@ get_desktop_item_name (void)
 	g_string_append (gs, g_strdup_printf ("named=%s", file_is_named_locale));
 	g_free (file_is_named_locale);
 
-	look_in_folder_utf8 = gnome_file_entry_get_full_path (GNOME_FILE_ENTRY(interface.look_in_folder_entry), FALSE);
+	look_in_folder_utf8 = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (interface.look_in_folder_entry));
 	look_in_folder_locale = g_locale_from_utf8 (look_in_folder_utf8 != NULL ? look_in_folder_utf8 : "", 
 	                                            -1, NULL, NULL, NULL);
 	g_string_append (gs, g_strdup_printf ("&path=%s", look_in_folder_locale));
@@ -937,15 +991,18 @@ get_desktop_item_name (void)
 static void
 start_animation (void)
 {
-	gchar *title = NULL;
+	if (search_command.first_pass == TRUE) {
 	
-	title = g_strconcat (_("Searching..."), " - ", _("Search for Files"), NULL);
-	gtk_window_set_title (GTK_WINDOW (interface.main_window), title);
+		gchar *title = NULL;
 	
-	gtk_label_set_text (GTK_LABEL (interface.results_label), "");
-	gsearch_spinner_start (GSEARCH_SPINNER (interface.spinner));
+		title = g_strconcat (_("Searching..."), " - ", _("Search for Files"), NULL);
+		gtk_window_set_title (GTK_WINDOW (interface.main_window), title);
 	
-	g_free (title);
+		gtk_label_set_text (GTK_LABEL (interface.results_label), "");
+		gsearch_spinner_start (GSEARCH_SPINNER (interface.spinner));
+	
+		g_free (title);
+	}
 } 
 
 static void
@@ -988,8 +1045,8 @@ handle_popt_args (void)
 	}
 	if (PoptArgument.path != NULL) {
 		popt_args_found = TRUE;
-		gtk_entry_set_text (GTK_ENTRY(gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY(interface.look_in_folder_entry))),
-				    g_locale_to_utf8 (PoptArgument.path, -1, NULL, NULL, NULL));
+		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (interface.look_in_folder_entry),
+		                               g_locale_to_utf8 (PoptArgument.path, -1, NULL, NULL, NULL));
 	}	
 	if (PoptArgument.contains != NULL) {
 		popt_args_found = TRUE;	
@@ -1141,11 +1198,15 @@ handle_search_command_stdout_io (GIOChannel 	*ioc,
 					broken_pipe = TRUE;
 				}
 				else if (status == G_IO_STATUS_AGAIN) {
-					while (gtk_events_pending ()) {
-						if (search_data->running == MAKE_IT_QUIT) {
-							return FALSE;
-						}
-						gtk_main_iteration (); 				
+					if (gtk_events_pending ()) {
+						intermediate_file_count_update ();
+						while (gtk_events_pending ()) {
+							if (search_data->running == MAKE_IT_QUIT) {
+								return FALSE;
+							}
+							gtk_main_iteration (); 
+						}	
+ 				
 					}
 				}
 				
@@ -1214,11 +1275,14 @@ handle_search_command_stdout_io (GIOChannel 	*ioc,
 			g_timer_elapsed (timer, &duration);
 
 			if (duration > GNOME_SEARCH_TOOL_REFRESH_DURATION) {
-				while (gtk_events_pending ()) {
-					if (search_data->running == MAKE_IT_QUIT) {
-						return FALSE;
-					}
-					gtk_main_iteration ();		
+				if (gtk_events_pending ()) {
+					intermediate_file_count_update ();
+					while (gtk_events_pending ()) {
+						if (search_data->running == MAKE_IT_QUIT) {
+							return FALSE;
+						}
+						gtk_main_iteration ();		
+					}		
 				}
 				g_timer_reset (timer);
 			}
@@ -1230,35 +1294,51 @@ handle_search_command_stdout_io (GIOChannel 	*ioc,
 	}
 
 	if (condition != G_IO_IN || broken_pipe == TRUE) { 
+
+		g_io_channel_shutdown (ioc, TRUE, NULL);
 		
 		if (search_data->running == MAKE_IT_STOP) {
 			search_data->aborted = TRUE;
 		}
 		
-		search_data->lock = FALSE;
-		search_data->running = NOT_RUNNING;
-		search_data->not_running_timeout = TRUE;
-		g_hash_table_destroy (search_data->pixbuf_hash);
-		g_timeout_add (500, not_running_timeout_cb, NULL);
+		if ((search_data->aborted == FALSE)
+		     && (search_data->quick_mode == TRUE) 
+		     && (search_data->first_pass == TRUE) 
+		     && (search_data->disable_second_pass == FALSE)
+		     && (is_second_scan_excluded_path (search_data->look_in_folder) == FALSE)) { 
+		    
+			gchar *command;
+			
+			command = build_search_command (FALSE);
+			spawn_search_command (command);
+			
+			g_free (command);
+		}
+		else {
+			search_data->lock = FALSE;
+			search_data->running = NOT_RUNNING;
+			search_data->not_running_timeout = TRUE;
+			g_hash_table_destroy (search_data->pixbuf_hash);
+			g_hash_table_destroy (search_data->file_hash);
+			g_timeout_add (500, not_running_timeout_cb, NULL); 
 
-		update_search_counts ();
-		stop_animation ();
+			update_search_counts (); 
+			stop_animation ();
 
-		gtk_window_set_default (GTK_WINDOW(interface.main_window), interface.find_button);
-		gtk_widget_set_sensitive (interface.additional_constraints, TRUE);
-		gtk_widget_set_sensitive (interface.expander, TRUE);
-		gtk_widget_set_sensitive (interface.table, TRUE);
-		gtk_widget_set_sensitive (interface.find_button, TRUE);
-		gtk_widget_hide (interface.stop_button);
-		gtk_widget_show (interface.find_button);
+			gtk_window_set_default (GTK_WINDOW(interface.main_window), interface.find_button);
+			gtk_widget_set_sensitive (interface.additional_constraints, TRUE);
+			gtk_widget_set_sensitive (interface.expander, TRUE);
+			gtk_widget_set_sensitive (interface.table, TRUE);
+			gtk_widget_set_sensitive (interface.find_button, TRUE);
+			gtk_widget_hide (interface.stop_button);
+			gtk_widget_show (interface.find_button); 
 
-		/* Free the gchar fields of search_command structure. */
-		g_free (search_data->look_in_folder);
-		g_free (search_data->file_is_named_pattern);
-		g_free (search_data->regex_string);
-		g_free (search_data->date_format_pref);
-
-		g_io_channel_shutdown (ioc, TRUE, NULL);
+			/* Free the gchar fields of search_command structure. */
+			g_free (search_data->look_in_folder);
+			g_free (search_data->file_is_named_pattern);
+			g_free (search_data->regex_string);
+			g_free (search_data->date_format_pref);
+		}
 		return FALSE;
 	}
 	return TRUE;
@@ -1298,11 +1378,15 @@ handle_search_command_stderr_io (GIOChannel 	*ioc,
 					broken_pipe = TRUE;
 				}
 				else if (status == G_IO_STATUS_AGAIN) {
-					while (gtk_events_pending ()) {
-						if (search_data->running == MAKE_IT_QUIT) {
-							break;
+					if (gtk_events_pending ()) {
+						intermediate_file_count_update ();
+						while (gtk_events_pending ()) {
+							if (search_data->running == MAKE_IT_QUIT) {
+								break;
+							}
+							gtk_main_iteration ();
+
 						}
-						gtk_main_iteration ();
 					}
 				}
 				
@@ -1475,26 +1559,29 @@ spawn_search_command (gchar *command)
 		return;
 	}
 	
-	search_command.lock = TRUE;
-	search_command.aborted = FALSE;
-	search_command.running = RUNNING; 
-	search_command.pixbuf_hash = g_hash_table_new (g_str_hash, g_str_equal);
+	if (search_command.first_pass == TRUE) {
+		search_command.lock = TRUE;
+		search_command.aborted = FALSE;
+		search_command.running = RUNNING; 
+		search_command.pixbuf_hash = g_hash_table_new (g_str_hash, g_str_equal);
+		search_command.file_hash = g_hash_table_new (g_str_hash, g_str_equal);
 	
-	/* Get value of nautilus date_format key */
-	search_command.date_format_pref = gsearchtool_gconf_get_string ("/apps/nautilus/preferences/date_format");
+		/* Get value of nautilus date_format key */
+		search_command.date_format_pref = gsearchtool_gconf_get_string ("/apps/nautilus/preferences/date_format");
 
-	gtk_window_set_default (GTK_WINDOW(interface.main_window), interface.stop_button);
-	gtk_widget_show (interface.stop_button);
-	gtk_widget_set_sensitive (interface.stop_button, TRUE);
-	gtk_widget_hide (interface.find_button);
-	gtk_widget_set_sensitive (interface.find_button, FALSE);
-	gtk_widget_set_sensitive (interface.results, TRUE);
-	gtk_widget_set_sensitive (interface.additional_constraints, FALSE);
-	gtk_widget_set_sensitive (interface.expander, FALSE);
-	gtk_widget_set_sensitive (interface.table, FALSE);
+		gtk_window_set_default (GTK_WINDOW(interface.main_window), interface.stop_button);
+		gtk_widget_show (interface.stop_button);
+		gtk_widget_set_sensitive (interface.stop_button, TRUE);
+		gtk_widget_hide (interface.find_button);
+		gtk_widget_set_sensitive (interface.find_button, FALSE);
+		gtk_widget_set_sensitive (interface.results, TRUE);
+		gtk_widget_set_sensitive (interface.additional_constraints, FALSE);
+		gtk_widget_set_sensitive (interface.expander, FALSE);
+		gtk_widget_set_sensitive (interface.table, FALSE);
 
-	gtk_tree_view_scroll_to_point (GTK_TREE_VIEW(interface.tree), 0, 0);
-	gtk_list_store_clear (GTK_LIST_STORE(interface.model));
+		gtk_tree_view_scroll_to_point (GTK_TREE_VIEW(interface.tree), 0, 0);	
+		gtk_list_store_clear (GTK_LIST_STORE(interface.model));
+	}
 	
 	ioc_stdout = g_io_channel_unix_new (child_stdout);
 	ioc_stderr = g_io_channel_unix_new (child_stderr);
@@ -1691,7 +1778,7 @@ create_additional_constraint_section (void)
 
 	interface.add_button = gtk_button_new_from_stock (GTK_STOCK_ADD);
 	GTK_WIDGET_UNSET_FLAGS (interface.add_button, GTK_CAN_DEFAULT);
-	interface.constraint_size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+	interface.constraint_size_group = gtk_size_group_new (GTK_SIZE_GROUP_BOTH);
 	gtk_size_group_add_widget (interface.constraint_size_group, interface.add_button);
 	
 	g_signal_connect (G_OBJECT(interface.add_button),"clicked",
@@ -1869,7 +1956,6 @@ create_main_window (void)
 	GtkWidget	*vbox;
 	GtkWidget 	*entry;
 	GtkWidget	*label;
-	GtkWidget 	*folder_entry;
 	GtkWidget 	*button;
 	GtkWidget 	*window;
 
@@ -1921,8 +2007,8 @@ create_main_window (void)
 	if (GTK_IS_ACCESSIBLE (gtk_widget_get_accessible(interface.file_is_named_entry)))
 	{
 		interface.is_gail_loaded = TRUE;
-		add_atk_namedesc (interface.file_is_named_entry, NULL, _("Enter the file name you want to search"));
-		add_atk_namedesc (entry, _("Name Contains Entry"), _("Enter the file name you want to search"));
+		add_atk_namedesc (interface.file_is_named_entry, NULL, _("Enter the file name you want to search."));
+		add_atk_namedesc (entry, _("Name Contains Entry"), _("Enter the file name you want to search."));
 	}	
      
 	g_signal_connect (G_OBJECT (interface.file_is_named_entry), "activate",
@@ -1939,33 +2025,24 @@ create_main_window (void)
 	
 	gtk_table_attach(GTK_TABLE(interface.table), label, 0, 1, 1, 2, GTK_FILL, 0, 0, 0);
 	
-	interface.look_in_folder_entry = gnome_file_entry_new ("gsearchtool-folder-entry", _("Browse"));
-	gnome_file_entry_set_directory_entry (GNOME_FILE_ENTRY (interface.look_in_folder_entry), TRUE);
-	g_object_set (G_OBJECT (interface.look_in_folder_entry), "use-filechooser", TRUE, NULL);
-	
-	entry = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY(interface.look_in_folder_entry));
-	gtk_label_set_mnemonic_widget (GTK_LABEL(label), interface.look_in_folder_entry);
-	folder_entry = gnome_file_entry_gnome_entry (GNOME_FILE_ENTRY(interface.look_in_folder_entry));
+	interface.look_in_folder_entry = gtk_file_chooser_button_new (_("Browse"));
+	gtk_file_chooser_set_action (GTK_FILE_CHOOSER (interface.look_in_folder_entry), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
+	gtk_label_set_mnemonic_widget (GTK_LABEL(label), GTK_WIDGET (interface.look_in_folder_entry));		
 	gtk_table_attach (GTK_TABLE(interface.table), interface.look_in_folder_entry, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL | GTK_SHRINK, 0, 0, 0);
 	
 	if (interface.is_gail_loaded)
 	{ 
-		add_atk_namedesc (GTK_WIDGET(folder_entry), NULL, _("Enter the folder name where you want to start the search"));
-		add_atk_namedesc (GTK_WIDGET(entry), _("Look in Folder Entry"), _("Enter the folder name where you want to start the search"));
+		add_atk_namedesc (GTK_WIDGET(interface.look_in_folder_entry), NULL, _("Select the folder where you want to start the search."));
 	}
 	
-	g_signal_connect (G_OBJECT (entry), "key_press_event",
-			  G_CALLBACK (look_in_folder_key_press_cb),
-			  NULL);
-	
-	g_signal_connect (G_OBJECT (interface.look_in_folder_entry), "changed",
+	g_signal_connect (G_OBJECT (interface.look_in_folder_entry), "current-folder-changed",
 			  G_CALLBACK (constraint_entry_changed_cb),
 			  NULL);
 			  
 	locale_string = g_get_current_dir ();
 	utf8_string = g_filename_to_utf8 (locale_string, -1, NULL, NULL, NULL);
 	
-	gtk_entry_set_text (GTK_ENTRY(entry), utf8_string);
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (interface.look_in_folder_entry), utf8_string);
 	g_free (locale_string);
 	g_free (utf8_string);
 	
@@ -2097,7 +2174,7 @@ set_clone_command (gint *argcp, gchar ***argvp, gpointer client_data, gboolean e
 	g_free (tmp);
 	g_free (file_is_named_locale);
 
-	look_in_folder_utf8 = gnome_file_entry_get_full_path (GNOME_FILE_ENTRY(interface.look_in_folder_entry), FALSE);
+	look_in_folder_utf8 = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (interface.look_in_folder_entry));
 	look_in_folder_locale = g_locale_from_utf8 (look_in_folder_utf8 != NULL ? look_in_folder_utf8 : "", 
 	                                            -1, NULL, NULL, NULL);
 	g_free (look_in_folder_utf8);
@@ -2272,7 +2349,6 @@ main (int 	argc,
 	GnomeProgram *gsearchtool;
 	GnomeClient  *client;
 	GtkWidget    *window;
-	GtkIconInfo  *icon_info;
 
 	memset (&interface, 0, sizeof (interface));
 
@@ -2298,12 +2374,8 @@ main (int 	argc,
 					  GNOME_PARAM_APP_DATADIR, DATADIR,  
 					  NULL);
 	gsearchtool_init_stock_icons ();				  
-	icon_info = gtk_icon_theme_lookup_icon (gtk_icon_theme_get_default (), GNOME_SEARCH_TOOL_ICON, 
-	                                        GNOME_SEARCH_TOOL_DEFAULT_ICON_SIZE, 0);
-	if (icon_info) {
-		gnome_window_icon_set_default_from_file (gtk_icon_info_get_filename (icon_info));
-		gtk_icon_info_free (icon_info);
-	}
+
+	gtk_window_set_default_icon_name (GNOME_SEARCH_TOOL_ICON);
 
 	if (!bonobo_init_full (&argc, argv, bonobo_activation_orb_get (), NULL, NULL))
 		g_error (_("Cannot initialize bonobo."));
