@@ -1,11 +1,30 @@
-#include <config.h>
+/*  ----------------------------------------------------------------------
+
+    Copyright (C) 1998  Cesar Miquel  (miquel@df.uba.ar)
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+
+    ---------------------------------------------------------------------- */
+
+#include <libgnome/gnome-util.h>
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
+#include "actions.h"
 #include "logview.h"
 #include <string.h>
-#include <stdlib.h>
-#include <sys/types.h>
 #include <regex.h>
-#include <gnome.h>
-#include "monitor.h"
 
 #define MAX_NUM_MATCHES     10
 #define DELIM               ":"
@@ -23,15 +42,9 @@ static void add_actions_entry_cb (GtkWidget *widget, gpointer data);
 static void edit_action_entry (Action *action);
 static void set_atk_relation (GtkWidget *label, GtkWidget *widget);
 
-int exec_action_in_db (Log *log, LogLine *line, GList *db);
-int read_actions_db (char *filename, GList **db);
 int write_actions_db (char *filename, GList *db);
 
-
-
 extern GList *actions_db;
-
-
 static GList *local_actions_db = NULL;
 
 GtkWidget *actions_dialog = NULL;
